/*
args_in.c  B.R.Glasberg, Dept Exptl. Psychol., Cambridge.
parse command line (i.e sort options) for 'recruit.exe'
*-------------------------------------------------------

* get in input file					-i file
*	output file					-o file

*	dB for rms					-d dB
*	boost						-b
*       equal loudness                                  -e dB
* nal_calc must have the same equal loudness point as this program
*       amplify output                                  -a dB
this can be used to scale the output down e.g. to stop overflows


* if default rms & dB for rms are used, it is as though +- 10v (+- 32767)
* for a sinusoid gives an output of 102 dB SPL (Masscomp with 10dB attenuation)
* use rlevel to measure a file's level at this calibration
* if you put in your own calibration, the output file has that same input calibration
*/


#include "crecruit.h"

extern int verbose,over_recruited;

/* ---------------------------------------------------------------------------- */
void info(char** progname)
{

  fprintf(stderr," use: %s -i filein -n audfile [-o fileout_root] [-d dB_rms] [-e equal_loudness_dB]",*progname);
  fprintf(stderr," [-a amplify_output_dB] [-b] [-v]\n");

  fprintf(stderr,"\n audfile is a file derived from the audiogram containing powers & NAL gains");

  fprintf(stderr,"\n where dB_rms [%5.1f] is that given by a sinusoid of maximum level (+-32767)\n",dB_rms_const);
  fprintf(stderr," The defaults are right for Sennheisers 414 through a 10 dB attenuator\n");
  fprintf(stderr," -e dB sets the point where impaired catches up with normal [default %5.1f]\n",eq_loud_const);
  fprintf(stderr,"!!! if the equal loudness point is changed in this program !!!\n");
  fprintf(stderr,"!!! it must be changed to the same value in program nal_calc !!!\n");

  fprintf(stderr," -a dB amplifies / decreases the output level\n");
  fprintf(stderr," -b causes the input to be boosted by the levels in the output file from nal_calc\n");
  fprintf(stderr," -v is verbose: prints out a few things\n");
  fprintf(stderr," If an outfile_root is not used, the infile name plus '.r' is used\n");
}

/* ---------------------------------------------------------------------------- */

/* FUNCTION args_in() */

void args_in(int argc,char **argv,char *infile,char *outfile,char *audfile, float *rms,float* dB_rms,float* eq_loud_dB,short* doboost,float* amplify_dB)
{
long infileflag = FALSE, outfileflag = FALSE, audfileflag = FALSE;
int c;
extern char *optarg;
extern int optind;

/* set up defaults */
  verbose        = FALSE;
  over_recruited = FALSE;
  *eq_loud_dB = eq_loud_const;
  *doboost = FALSE;
  *dB_rms  = dB_rms_const;
  *rms     = rms_const;  /* this is now not a changeable parameter, dB_rms can be altered */
  *amplify_dB = 0.0;

  while ((c = getopt(argc, argv, "Ibvi:o:n:d:e:a:")) != EOF)
    switch (c) {
    case 'b': *doboost = TRUE; break;
    case 'v': verbose = TRUE; break;
    case 'i': strcpy( infile,optarg); infileflag = TRUE; break;
    case 'o': strcpy(outfile,optarg); outfileflag = TRUE; break;
    case 'n': strcpy(audfile,optarg); audfileflag = TRUE; break;
    case 'd': *dB_rms = atof(optarg); break;
    case 'e': *eq_loud_dB = atof(optarg); break;
    case 'a': *amplify_dB = atof(optarg);break;
    case 'I': info(&argv[0]); exit(0);
    case '?': fprintf(stderr,"-%c is not an allowed command line switch\n",c); exit(1);
  }

  if(!infileflag || !audfileflag){
    fprintf(stderr,"\n\n !!!! need at least an input file (which is an output from f_align) !!!!\n");
    fprintf(stderr," !!!! and an audfile (which comes from program nal_calc)            !!!!\n\n");
    info(&argv[0]);
    exit(1);
  }

  if(!outfileflag) {
    strcpy(outfile,infile);
    strcat(outfile,".r");
  }
  fprintf(stderr," input file      : %s\n", infile);
  fprintf(stderr," outfile         : %s\n",outfile);

  if(verbose) {
    fprintf(stderr," dB_rms                   : %6.1f\n",*dB_rms);
    fprintf(stderr," equal loudness occurs at : %5.1f dB\n",*eq_loud_dB);
    fprintf(stderr," output amplified by      : %5.1f dB\n",*amplify_dB);
  }
  if(*doboost) fprintf(stderr," boosted\n");
  if(*dB_rms <= 0.0) fprintf(stderr," !!! is dB_rms (%.1f) right?  !!!\n",*dB_rms);
}
