/* COMPLEX.C to write a simple binary file for input to
 recruitment simulation software.  Generates harmonic complex 
 from equal amplitude sine waves, NCOMPS harmonics, starting from 
 F0.  Normalises output so that |max| is 10,000.
B.R.Glasberg, Cambridge, 1993-1995
*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#define PI     3.14159265
#define TWOPI  (2 * PI)
#define CLK    16000
#define NPTS   (CLK * 4)  /* just a few seconds */
#define SHORTS sizeof(short)
#define NCOMPS 1 /* number of components in complex harmonics */
#define F0 1000. /* starting harmonic frequency */

void main(int argc, char** argv)
{

float wave[NPTS], comp, max = 0.0;
int i,n;
short i2wave[NPTS];
static char outfile[] = "complex.wav";
FILE *fd_out;

  for(n = 0 ; n < NPTS; n++) wave[n] = 0.0;

  for ( i = 0 ; i < NCOMPS ; i++){       /* variable number of components */
    comp = (i+1) * F0;                   /* start frequency */
    for (n = 0 ; n < NPTS ; n++){
      wave[n] += sin(comp * TWOPI * n / CLK);
    }
  }

  for ( n = 0 ; n < NPTS ; n++)
    if(fabs(wave[n]) > max) max = fabs(wave[n]);

  for ( n = 0 ; n < NPTS ; n++)
    i2wave[n] = (wave[n]/max) * 10000;

  if ((fd_out = fopen(outfile,"wb")) == NULL) {
    fprintf(stderr,"\n%s: Can't open OUTPUT file: %s\n",argv[0], outfile);
    exit(1);
  }

    fwrite(i2wave,SHORTS,NPTS,fd_out);
    fprintf(stdout,"\nWritten file [%s] with %2d harmonics\n", outfile,NCOMPS);
}
