/*
  ====================================================================
  filtunix.c: C code for recursive filter section. Dumb UNIX version
  ====================================================================

    M.A.Stone August 1990.
    This routine implement 4 stages of second order IIR filter as 4 identical
    stages in series.  (Same coefficients).
    The state vector for each filter bank is shortened since
    output states of previous stage are input states of next stage. ie
    X'n = Yn, and X'n-1 = Yn-1. State vector is in fact:
    (Xn-1,Yn-2,Yn-1,Y'n-2,Y'n-1,Y"n-2,Y"n-1,...etc...) State vector is updated
    continuously through each filter section, so Yn-2,Yn-1 before pointer are
    really Yn-1 and Yn for that clock period; hence te use as input for the
    next stage.

This version (18-04-91) for UNIX to remove possible compiler ambiguities.
(History: Masscomp 'C' compiler could not handle multiple '=' on same line:
it did not report an error but produced wrong code: hence unsophisticated coding)

Recruitment simulation: B.R.Glasberg 1993-1995
This version (15-01-93) for wide filter use with additional 2 pole HP Bessel
to control LF tail of some of the BP filters.  Coefficients set externally and
delays added to band-pass delays already calculated.
*/
#include "recwide4.h"
#include "proto.h"
/* external data areas defined in main program */
extern struct Biquad filter_coeffs[];/* bank of filter coeffcnts structs */
extern struct Hpbessel hpcoeffts[];/* bank of filter coeffcnts structs */
extern Filter_State  bank_state[N_OF_CHANS][XiYj]; /* current state of filter bank */

/*---------------------------------------------------------------------------*/
void aud_filt_bank(float *inp, float *outp, long numb) /* auditory filtering */
/*-------------------------------------------------------*/
/* inp:  pointer to current sample in buffer length numb */
/* outp: array to be filled with filter op: size >= (numb*N_OF_CHANS)*/
/* numb: number of samples to process */
{
  Filter_State *ptr2, *ptr1;
  float *xn, sum, wn, a0, a1, b1, b2, *lcl_op; /* inner loop vars */
  float *hpvct0, *hpvct1, *hpcfs, hpb1, hpb2, hpa0, hpa1, hpa2, tmp0, tmp1;
  long  indx, channel;
  struct Biquad *coeffs_i; /* pointer to current bp filter coeffs */
  Filter_State  *curr_vect;

/* for all channels: outer loop variables not register; inner loop are */

  for( channel = 0; channel < (N_OF_CHANS); channel++) {
/*    if(channel%4 == 0) printf(".");fflush(stdout);  */
    coeffs_i = filter_coeffs+channel;
    curr_vect = bank_state[channel]; /* current state vector */
    a0 = coeffs_i->k0num ; /* use dsp notation: numerator coeffs are */
    a1 = coeffs_i->k1num ; /* ai and denominator are bi. */
    b1 = coeffs_i->k1denom;
    b2 = coeffs_i->k2denom;
/* extra code to pick up coefficients for 2 pole high-pass Bessel filters */
    if(channel >= HPSTRT){ /* choose correct HP filter and state vector */
      hpvct0 = iirvect + N_HPSTS*(channel-HPSTRT);
      hpvct1 = hpvct0 + 1;


/* ----------------------------------------------------------------------------
      hpcfs = iircfs + N_HPCFS*(channel-HPSTRT);
      hpb1 = *(hpcfs++); hpb2 = *(hpcfs++);
      hpa0 = *(hpcfs++); hpa1 = *(hpcfs++); hpa2 = *hpcfs;
---------------------------------------------------------------------------- */


      hpa0 = hpcoeffts[channel-HPSTRT].a0;
      hpa1 = hpcoeffts[channel-HPSTRT].a1;
      hpa2 = hpcoeffts[channel-HPSTRT].a2;
      hpb1 = hpcoeffts[channel-HPSTRT].b1;
      hpb2 = hpcoeffts[channel-HPSTRT].b2;
    }
    xn = inp; /* reset working pointer to start of input buffer each pass */
    lcl_op = outp + channel;

/* filter is 4 identical sections concatenated, S/(S*S + a*a); & S = (s+ERB) */
  for( indx = numb ; indx-- > 0; ) {    /* process buffer of length numb */
      ptr1 = curr_vect; /* point to relevant state vector, each sample */

  /* Sect 1 */
      sum = *(xn++);
      ptr2 = ptr1;
      sum -= b1* *(ptr1++);
      sum -= b2* *ptr1;
      wn = sum; /* temp variable */
      sum *= a0;
      sum += a1* *ptr2; /* update Yn-1 to Yn-2 */
      *ptr1 = *ptr2;
      *ptr2 = wn;
      ptr1++;
/* Sect 2 */
      ptr2 = ptr1;
      sum -= b1* *(ptr1++);
      sum -= b2* *ptr1;
      wn = sum; /* temp variable */
      sum *= a0;
      sum += a1* *ptr2; /* update Yn-1 to Yn-2 */
      *ptr1 = *ptr2;
      *ptr2 = wn;
      ptr1++;
/* Sect 3 */
      ptr2 = ptr1;
      sum -= b1* *(ptr1++);
      sum -= b2* *ptr1;
      wn = sum; /* temp variable */
      sum *= a0;
      sum += a1* *ptr2; /* update Yn-1 to Yn-2 */
      *ptr1 = *ptr2;
      *ptr2 = wn;
      ptr1++;
/* Sect 4 */
      ptr2 = ptr1;
      sum -= b1* *(ptr1++);
      sum -= b2* *ptr1;
      wn = sum; /* temp variable */
      sum *= a0;
      sum += a1* *ptr2; /* update Yn-1 to Yn-2 */
      *ptr1 = *ptr2;
      *ptr2 = wn;
      ptr1++; /* BG: this line should be redundant MAS Jan 93 */
/* Here's the 2 pole IIR filter.  Use different variable names for clarity: 'sum' carries value in and out */
      if(channel >= HPSTRT){
/* Direct form 2 IIR filter (Rabiner and Gold DSP book) */
	tmp0  = sum - hpb1 * *hpvct0; /* b1 term */
	tmp0 -=       hpb2 * *hpvct1; /* b2 term */
	tmp1  = hpa0 * tmp0;    /* a0 term */
	tmp1 += hpa1 * *hpvct0; /* a1 term */
	tmp1 += hpa2 * *hpvct1; /* a2 term */
	*hpvct1 = *hpvct0; /* update state vector every sample */
	*hpvct0 = tmp0;
	sum = tmp1; /* is output sample */
      }
      *lcl_op = sum; /* Final Yn: put in output buffer */
      lcl_op += N_OF_CHANS; /* increment through output array by time slot */
    }
  }
}
/* ----------  end of filtunix.c  ------------------- */
