/*
  INITIAL4.C subroutines to initiate the filter states and coefficient values.
    M.A.Stone August 1990

  Assumes that there will be N_OF_CHANS filters in use, starting
from LOWEST_CF and ending at HIGHEST_CF. Calculate the E numbers of these
filters and space the rest equally along the E scale. If these figures become
variable at a later date, then make them extern static in rec_wide4.h.

This version has filter centre frequencies from 100 Hz to 5836.7 Hz, this is
'normal' Erb number 3.36 to 30.36.
The erbs are assumed to be 3 times wider than normal

The spacing is 2.25 'normal' erbs, i.e. centre freqs are 3, 5.25, 7.5, 9.75 ... 30 'normal' erbs

B.R.Glasberg, Dept. Exp. Psych., Cambridge. 1993-1995*/

#include "recwide4.h"
#include "proto.h"

extern struct Hpbessel hpcoeffts[];/* bank of filter coeffcnts structs */

float  E_low, E_high, delta_E, current_E, erb, freq_at_E;
struct Biquad *this_filt;

/*------------------------------------------------------------------------------*/
void calculate_filter_data(struct Biquad *coeffs, int rcd)/* fill up biquadratic*/
{
long channel;
float erbT;
complex cf_num, cf_denom, Tf, var1, var2, var3;
FILE *bq_file;

#define PRT 0

  E_low = ErbScale (LOWEST_CF);
  E_high= ErbScale(HIGHEST_CF);


  delta_E = (E_high - E_low) / (float)(N_OF_CHANS-1);
  current_E = E_low;

  this_filt = coeffs;   /* use local pointer for legibility */

#if PRT
  printf("E_low:  %f    E_high:  %f\n\n",E_low,E_high);
  printf("\n Chan  Freq  Dly:Chan  Freq  Dly:Chan  Freq  Dly:Chan  Freq  Dly");
#endif

  for(channel = 0; channel < N_OF_CHANS; channel++){
    this_filt->E_no = current_E;
    freq_at_E = ( FofErbScale(current_E) );
    erb = Erb(freq_at_E);
    erbT= (TwoPi*erb)/SAMPLE_CLK;
    erbT *= BROADENING; /* n stages in cascade; hence wider individual */

    this_filt->k1num   = exp( -erbT) * cos(TwoPi * freq_at_E/SAMPLE_CLK);
    this_filt->k1denom = -2. * this_filt->k1num;
    this_filt->k2denom =  exp(-2.* erbT);
    this_filt->k1num   = -this_filt->k1num; /* response is later normalised */
    this_filt->k0num   =  1.0;    /* 2.*erbT */
    this_filt->delay   = (DELAY_SF/erbT) + 0.5;

    var2.re = this_filt->k1num; var2.im = 0.0;
    var3.re =  cos(TwoPi*freq_at_E/SAMPLE_CLK);
    var3.im = -sin(TwoPi*freq_at_E/SAMPLE_CLK);
    cmplxmult(&var2,&var3,&var2); /* a1*exp(-sT) */

    var1.re = this_filt->k0num; var1.im = 0.0;
    cmplxadd (&var1,&var2,&cf_num); /* a0+a1*exp(-sT) */

    var2.re = this_filt->k1denom; var2.im = 0.0;
    cmplxmult(&var2,&var3,&var2); /* b1*exp(-sT) */
    var2.re += 1.0; /* 1+b1*exp(-sT) */

    var3.re =  cos(2.*TwoPi*freq_at_E/SAMPLE_CLK);
    var3.im = -sin(2.*TwoPi*freq_at_E/SAMPLE_CLK);
    var1.re = this_filt->k2denom; var1.im = 0.0;
    cmplxmult(&var3,&var1,&var1); /* b2*exp(-2sT) */

    cmplxadd (&var1,&var2,&cf_denom); /* sigma(bi*exp(-isT) */

    if(!cmplxdiv(&cf_denom,&cf_num,&Tf)){ /* invert to get normalised values */
	printf("\n***Filter coefficients failed to initialise ***\n\n"); exit(0); }
    cmplxmag(&Tf,&Tf); this_filt->k1num *= Tf.re; this_filt->k0num *= Tf.re;


#if PRT
    if(!(channel%4))printf("\n");
/*    printf("[%2d] %10.4f", channel, Tf.re);  */
    printf(" [%2d] %6.1f %3d",channel,freq_at_E,this_filt->delay);
    fflush(stdout);
#endif

    current_E += delta_E;
    this_filt++;   /* use local pointer for legibility */
  }
  if(rcd){ /* make file copy of structures in biquad.tst    */
    bq_file = fopen("biquad.tst","wt"); /* write text file for history  */
    channel = wr_biquad_file( bq_file, coeffs, N_OF_CHANS);
    if(channel != N_OF_CHANS) printf("\nDid not write full file\n");
/*    else printf("\nFile of filterbank data written as BIQUAD.TST\n");*/
    fclose(bq_file);
  }
}
/*-----------------------------------------------------------------------*/
void intlse_filter_states(Filter_State *vect) /* initialise Xn and Yns of filter banks */
/* Every filter has XiYj states stored: 'vect' is of type 'pointer to float'*/
{
 long channel, state_no;
  for(channel  = N_OF_CHANS; channel-- > 0; )
    for(state_no = XiYj; state_no-- > 0; ) *(vect++) = 0.0;
}
/*-----------------------------------------*/
void zero_array(float *elemnt,long howmany)
/*--- Initialise an array of float to zero */
{
long i;
for(i = howmany; i-- > 0; ) *(elemnt++) = 0.0;
}
/*-------------------------------------------------------------*/
void convert_to_float(short *input, float *output, long howmany)
/*------------------------------- convert input array to float */
{
long i;
for(i = 0; i++ < howmany; ) *(output++) = (float) *(input++);
}
/*-------------------------------------------------------------*/
void convert_to_short(float *input,short *output,long howmany)
/*------------------------------- convert input array to short */
{
long i;
  for(i = 0; i++ < howmany; )
    *(output++) = (short) *(input++);
}
/* ----------------  end of initial.c  -------------------- */

/* ------------- 2 pole hp bessel filter stuff follows ------------------ */
#define PI 3.1415926538

void error1(char *msg)

{ printf("Fails at %s",msg); exit(0); }
/* -------------------------------------------------------------------- */
void calc_hp_coeffts(struct Hpbessel *coeffts)
/* 5 channels 8-12, a0,a1,a2,b1,b2 */
{
static float cornerfq[] = {0.2, 0.4, 1.2, 2.2, 3.3}; /* channels 8 - 12 incl.*/
float A,A0,A1,A2,B0,B1,B2,C;
float a0,a1,a2,b0,b1,b2,fs; /* parameters used in z plane transformation */
float fc; 
/* filter normalised frequency parameters used for defining response curve */
float ai_tot, bi_tot, Tf;
long channel;

  fs = SAMPLE_CLK/1000.; /* sampling freq in kHz */

  B0 = 1.; B1 = 3.; B2 = 3.; /* 2 pole HighPass Bessel */
  A0 = 0.; A1 = 0.; A2 = 3.;

  for( channel = 0 ; channel < No_to_HPass ; channel++ ) {
    fc = cornerfq[channel];
    C = 1./tan((PI*fc)/fs); /* true bilinear transform eq 7-8, p172*/
    A  = (B0 + B1*C + (B2*C*C));
/* DSP, William D.Stanley, Reston press, pp 174-175 */
    a0 = (A0 + A1*C + A2*C*C)/A;
    a1 = 2.*(A0 - A2*C*C)/A;
    a2 = (A0 - A1*C + A2*C*C)/A;
    b0 = 1.0;
    b1 = 2.*(B0 - B2*C*C)/A;
    b2 = (B0 - B1*C + B2*C*C)/A;

/* now normalise coefficents at z=exp(s0T), s0 = Nyquist limit (alternate sign each sample) */
    ai_tot = a0; ai_tot -= a1; ai_tot += a2;
    bi_tot = 1.; bi_tot -= b1; bi_tot += b2;
    Tf = bi_tot/ai_tot; /* calculate normalisation factor */
    a0 *= Tf; a1 *= Tf; a2 *= Tf;
    coeffts[channel].a0 = a0;
    coeffts[channel].a1 = a1;
    coeffts[channel].a2 = a2;
    coeffts[channel].b1 = b1;
    coeffts[channel].b2 = b2;
/*
To work out the delay of this HP filter, courtesy of M.A.Stone is :-
tan phi = -3*fp/fc / (1 - 3*(fp/fc)^2)
(fp = frequency where delay is to be calc, fc = corner freq)
delay = phi/2pi * fs/fp  (fs = sample freq = 16kHz)
e.g. channel 8
fc = 0.2, fp = 2.066, fs = 16, tan phi = (-3*2.066/0.2)/(1-3*(2.066/.2)^2)
= 31/-319.1 =  = 0.97.  phi = tan-1 .097 (radians) = .097
delay = .097/2pi * 16/2.066 = 0.12 clock units
channel 9 delay = .14, channel 10 = .25, channel 11 = .273, channel 12 = .246
Thus the delay for the 5 channels used can all be called zero.
*/
  }
}
