/* nrutil.c from Numerical Recipes in C: memory allocation for matrices */
/* probably need free matrix & free smatrix sometime */ 

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>

#define NR_END 1
#define FREE_ARG char*

void nrerror(char error_text[])
{
fprintf(stderr,"%s\n",error_text);
exit(1);
}

long *lvector(long nl, long nh)
{
  long *v;
  v=(long *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(long)));
  if(!v) nrerror("allocation error in svector()");
  return v-nl+NR_END;
}

short *svector(long nl, long nh)
{
  short *v;
  v=(short *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(short)));
  if(!v) nrerror("allocation error in svector()");
  return v-nl+NR_END;
}

float *vector(long nl, long nh)
{
  float *v;
  v=(float *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float)));
  if(!v) nrerror("allocation error in vector()");
  return v-nl+NR_END;
}

unsigned char *cvector(long nl, long nh)
{
  unsigned char *v;
  v=(unsigned char *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(unsigned char)));
  if(!v) nrerror("allocation error in cvector()");
  return v-nl+NR_END;
}

float **pvector(long nl, long nh)
{
  float **v;
  v=(float **)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float **)));
  if(!v) nrerror("allocation error in pvector()");
  return v-nl+NR_END;
}

/* ==================================================== */

float **matrix(long nrl,long nrh, long ncl, long nch)
{
  long i, nrow=nrh-nrl+1, ncol=nch-ncl+1;
  float **m;

  m=(float **) malloc((size_t)((nrow+NR_END)*sizeof(float*)));
  if(!m) nrerror("allocation error in matrix()");
  m += NR_END;
  m -= nrl;
/* allocate rows and set pointers to them */
  m[nrl] = (float *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(float)));
  if(!m[nrl])  nrerror("allocation error 2 in matrix()");
  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for(i=nrl+1; i <= nrh; i++) m[i]=m[i-1]+ncol;
/*  return pointer to array of pointers to rows */
  return m;
}

/* ==================================================== */

short **smatrix(long nrl,long nrh, long ncl, long nch)
{
  long i, nrow=nrh-nrl+1, ncol=nch-ncl+1;
  short **m;

  m=(short **) malloc((size_t)((nrow+NR_END)*sizeof(short*)));
  if(!m) nrerror("allocation error in matrix()");
  m += NR_END;
  m -= nrl;
/* allocate rows and set pointers to them */
  m[nrl] = (short *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(short)));
  if(!m[nrl])  nrerror("allocation error 2 in matrix()");
  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for(i=nrl+1; i <= nrh; i++) m[i]=m[i-1]+ncol;
/*  return pointer to array of pointers to rows */
  return m;
}

void free_vector(float *v, long nl, long nh)
{
  free ((FREE_ARG) (v+nl-NR_END));
}
