/* outbits.c */
#include "crecruit.h"

#define n2av  161
#define av2   ((n2av+1)/2)

extern int verbose, over_recruited;

void outbits(float* data, float* wave, float *wave_out,float* n_minus1, float* boost, short channum,float* env,short doboost,float scale_factor)
{
int i, counter, i_av2_1;
float *real_phi, *outsig;
float av_sum, smooth_env[maxfftpts], smooth_env_0[maxfftpts];
float lin_boost,temp,adjust;

/*
* the following (n-1) table was calc. from the specified absolute thresholds 
* see M&G JASA 94 p2054 (1993) Table I
*/

/* **********************************************************
* work out envelope & normalize
* do all envelope bits for middle 48000 +- av2 points

* The envelope is the sqrt((Analytic signal)**2 + (original wave)**2)
* The fine structure is the (original wave/envelope)
* Reconstruction = envelope * fine_structure = env * original_wave/env = original_wave

* In the old program, fine_structure = cos(atan2(x,y)) /| <--- the angle
* = cos(The angle whose tan is x/y)                   / |y
*                                                    /  |
*                                                   -----
*						     x

* = y/sqrt(x*x + y*y)   = (if x = original wave * y = analytic signal)  original_wave/envelope
************************************************************ */

  real_phi =  env;   /* using the space of env & data */
  outsig   =  data;  /* for real_phi & outsig */

  av_sum = 0.0;
  counter = 0;
  for( i = offset-av2 ; i<= npts+offset+av2+1 ; i++){
    env[i] = sqrt(pow(data[i],2) + pow(wave[i],2)); /* ! last use of data */
    av_sum +=  env[i];
    counter++;
    if(counter < n2av) continue;
    i_av2_1 = i - av2 + 1;
    smooth_env_0[i_av2_1] = av_sum/n2av;
    if(smooth_env_0[i_av2_1] <= 0.0) {
      if(verbose)  fprintf(stderr," smooth_env_0 = %6.3f set to 1.0e-10",smooth_env_0[i_av2_1]);
      smooth_env_0[i_av2_1] = 1.0e-10;
    }
    av_sum -=  env[i-n2av+1];
  }

#if 0
  if(verbose) {  /* print middle few points of the envelope to the screen */
    printf("channel %2d:  ",channum);
    for(i=4086; i<=4096; i++) printf(" %6.4f",smooth_env_0[i]);
    printf("\n\n");
  }
#endif
/* ================================================================
this is the last use of array env() and data()

* Now have envelope and smoothed envelope in env and smooth_env_0
* only smooth_env_0 is used from now, env() is equivalenced with real_ph()
=================================================================== */

/* calc the (boosted) smoothed envelope for the particular channel and condition. */
    lin_boost = 1.0;

/* if the simulation requires that the input wave has been amplified before reaching
* the ear, that amplification would result in the envelope being increased by the boost
* and the fine structure also being amplified by the amount of boost applied
*/
    if(doboost) lin_boost=pow(10.,(boost[channum]/20.));

/* calc the phase - this is the fine structure of the waveform

* N.B. The fine structure is the (input waveform)/envelope, so the variable real_phi()
* is, in fact, the fine structure * (unsmoothed) envelope
*.................................................................................. */
    for ( i = 1+offset; i <= npts+offset; i++)
      real_phi[i] = wave[i] * scale_factor * lin_boost;

/* ................................................................................. */
    for ( i = 1+offset; i <= npts+offset; i++)
      smooth_env[i] = smooth_env_0[i] * lin_boost;

/* ................................................................................. */

    if(n_minus1[channum] == 0.0){    /* normal hearing for that channel    */
      for ( i = 1+offset; i <= npts+offset ; i++)/* could go through temp**(n-1) = 1.0 */
        outsig[i] = real_phi[i];                 /* without this if() else statement   */

    } else {
      for ( i = 1+offset; i <= npts+offset; i++) {
	temp = smooth_env[i];
	if(temp <= 1.0) {     /*	! recruitment   */
          outsig[i] = real_phi[i] * pow(temp,n_minus1[channum]);
        } else {

          outsig[i] = real_phi[i]; /* ! no recruitment */
          over_recruited = TRUE;
          if(verbose)fprintf(stderr," Over recruited\n");
        }
      }
    }


  if(n_minus1[channum] >= 2.0) {
    adjust = 0.0;
  } else {
    adjust = (2.0 - n_minus1[channum]) * 2.4;
  }

/*
fprintf(stderr,"outbits:  channum %2d,  n_minus1 %4.2f  adjust %4.2f\n",channum,n_minus1[channum],adjust);
*/

adjust = pow(10.0,(-adjust/20.0));
/*fprintf(stderr,"adjust = %6.2f\n",adjust);*/

  for (i = 1+offset; i <= npts+offset; i++ ) wave_out[i-offset] += (adjust * outsig[i]);

}  /* end of outbits, return npts real points in wave_out */
/*------------------------------------------------------------------------------*/
