/* recruit.c   for recruitment simulation */
/* B.R.Glasberg 1993-1995 */

#include "crecruit.h"
#define NRAMP   320 /* 320 points @ 16k = 20-ms ramps */
#define NSEEK   (-NRAMP * nchans * 2) /* number of bytes in 13 channels of ramp */
#define NWRITE  (npts - 2 * NRAMP)
#define ENDPTS  (npts - NRAMP)

/* global logical variables */
int verbose,over_recruited;

/* npts = 6000 = .375 secs in crecruit.h */

main(int argc, char **argv)
{
long realft_n;
long i,pts_read,last_pts, block =0, npts13 = npts * nchans;
float *wave, *data, *env, *wave_out, temp, max;
float rms,scale_factor,eq_loud_dB,eq_rms,amplify_dB, dB_rms;
float *n_minus1, *boost;
short *bufin13, *i2wave, *bufout, *ramp, doboost, channum, overflow;
short first = TRUE, last = FALSE;
char infile[50], outfile[50], ovfile[50], audfile[50];
FILE *fd_in, *fd_out, *fd_ov, *fd_aud;

  wave     =  vector(1l,(long) maxfftpts); /* input wave after scaling */
  data     =  vector(1l,(long) maxfftpts); /* analytic signal (hilbert) */
  env      =  vector(1l,(long) maxfftpts); /* envelope */
  wave_out =  vector(1l,(long) npts);      /* output buffer, one condition at a time */
  bufin13  = svector(1l,(long) npts13);    /* input buffer for all 13 channels */
  i2wave   = svector(1l,(long) npts);      /* input buffer, 1 channel at a time */
                                           /* & output waveform buffer, whole signal */
  boost    = vector(1l,(long)nchans);      /* NAL gain if required - read in from a file */
  n_minus1 = vector(1l,(long)nchans);      /* (power-1) to which envelope is raised */
                                           /* the program nal_calc produces the file */

/* Each 6000 points has 320 point linear ramps added at each end for overlap add */
  ramp     = svector(1l,(long) NRAMP);     /* holds ramped end across input blocks */
  bufout   = svector(1l,(long) npts);

/* Get command line parameters in */

	args_in(argc, argv, infile, outfile, audfile, &rms, &dB_rms, &eq_loud_dB, &doboost, &amplify_dB);

/*----------------------------*/
	eq_rms = rms * pow(10.0,(eq_loud_dB - dB_rms)/20.0);

/*
 scale_factor is the peak value that a sinusoid at the equal loudness point would have
 If the input is a sinusoid at level eq_loud_dB, the extracted envelope would be a straight
 line of amplitude 1.0
*/
  scale_factor = eq_rms * sqrt(2.0);
  if(verbose) fprintf(stderr," eq_rms = %8.2f scale_factor = %8.2f\n",eq_rms,scale_factor);

/* open input file */
	if ((fd_in = fopen(infile,"rb")) == NULL) {
    fprintf(stderr,"\n%s: Can't open INPUT file: %s\n\n",argv[0],infile);
    exit(1);
  }

/* open audfile */
  if ((fd_aud = fopen(audfile,"rt")) == NULL) {
    fprintf(stderr,"\n%s: Can't open audfile: %s\n\n",argv[0],audfile);
    exit(1);
  }

/* read audfile into n_minus1 & boost, the file contains ns so convert to n_minus1s */
  for( i = 1 ; i <= nchans ; i++){
    fscanf(fd_aud,"%f %f",&n_minus1[i],&boost[i]);
    n_minus1[i] -= 1.0;
    if(verbose) fprintf(stderr,"%5.2f %5.0f\n",n_minus1[i],boost[i]);
  }

	if ((fd_out = fopen(outfile,"wb")) == NULL) {
    fprintf(stderr,"\n%s: Can't open OUTPUT file: %s\n\n",argv[0],outfile);
    exit(1);
  }

  while(!last){

/* initialise output buffer used in outbits() to 0.0 */
    for ( i = 1 ; i <= npts ; i++) wave_out[i] = 0.0;

    block++; /* just a counter */
		if(verbose)printf("block %ld\n",block);
/* READ DATA IN: the data is read in in 6000 point bits (.375 sec at 16k) */
    if ((pts_read = fread(&bufin13[1],SHORTS,npts13,fd_in)) != npts13){
			fprintf(stderr,"\nEnd of file %s %ld points read\n",infile,pts_read);
			fprintf(stderr,"filling up with %ld zeros\n",npts13-pts_read);
			fprintf(stderr," for %ld points per channel\n",(npts13-pts_read)/nchans);
      for (i=pts_read+1; i <= npts13 ; i++) bufin13[i] = 0;
      last_pts = pts_read/nchans - NRAMP;
      last = TRUE;
    }

/* move reading point backwards 20-ms = 320 * 13 points = 4160 = 8320 bytes */
    if(!last){
      fseek(fd_in,(long)NSEEK,SEEK_CUR);
/*      fprintf(stderr,"ftell returns %ld\n",ftell(fd_in)); */
    }

/* copy channels 1-13 in order into i2wave */
    for (channum = 1 ; channum <= nchans ; channum++){
      if(verbose)printf("channel %d\n",channum);
      for ( i = 1 ; i <= npts ; i++) i2wave[i] = bufin13[(i-1)*nchans+channum];

/* i2wave contains 6000 points of channel channum */
/* process all 13 channels, adding them together into bufout */

/*			if(verbose)printf("going into int_to_real\n");*/
      int_to_real(wave,data,i2wave,scale_factor);

/* do fft on normalised waveform (see Press p 399) */

			 realft_n = npfft/2;
       realft(data,realft_n,1);

/* hilbert transform the spectrum */
/*    if(verbose)printf("going into hil_ph\n");*/
       hil_ph(data);
/* finish off inverse fft (Numerical Recipes) to get analytic signal */
/*	if(verbose)printf("going into realft_n\n");*/
       realft(data,realft_n, -1);
       for( i = 1; i <= npfft ; i++) data[i] /= realft_n; /* all real values */
/*  if(verbose)printf("going into outbits\n"); */
       outbits(data,wave,wave_out,n_minus1,boost,channum,env,doboost,scale_factor);
    } /* channum = 1 to nchans */

/*-----------------------------*/

    if(over_recruited)
        fprintf(stderr," Level exceeded %.1f dB in block %ld\n",eq_loud_dB,block);

    if(amplify_dB){
      temp = pow(10.0,(amplify_dB/20.0));
      printf(" AMPLIFYING\ temp = %f\n",temp);
      for ( i = 1 ; i <= npts ; i++) wave_out[i] *=  temp;
    }

/* wave_out is a float array of the output, convert to short for writing to file */

/* check for 16 bit overflow */
	max = 0.0;  
  for ( i = 1; i <= npts; i++ ){
      if(fabs(wave_out[i]) > max){
	max = fabs(wave_out[i]);
/*        if(verbose) fprintf(stderr," max: %f\n",max); */
      }
    }
		overflow = FALSE;
    if(max > 32767.4) {
			overflow = TRUE;
			max /= 32767.4;    /* max = proportion by which overflow has occurred */
			fprintf(stderr,"a: OVERFLOW max = %f\n",max);
    }

/* ---------------------- overflow -------------------------- */
    if(overflow) {
      fprintf(stderr,"b: OVERFLOW max = %f\n",max);
      fprintf(stderr,"%s: OVERFLOW :- divided by %8.4f = %6.1f dB\n",outfile,max,20.*log10(max));
      strcpy(ovfile,outfile); strcat(ovfile,"V");

/* open overflow report file */
      if ((fd_ov = fopen(ovfile,"at")) == NULL) {  /* append */
        fprintf(stderr,"\nCan't open OVERFLOW file: %s\n\n",ovfile);exit(1);
      }
      fprintf(stderr," overflow file %s written\n",ovfile);
			fprintf(fd_ov,"%s: OVERFLOW :- divided by %8.4f = %6.1f dB in block %ld\n",outfile,max,20.*log10(max),block);
      fclose(fd_ov);
    }
/* --------------- end of overflow -------------------------- */

    for( i = 1 ; i <= npts ; i++) {
      if(overflow) bufout[i] = wave_out[i]/max;
      else
      bufout[i] = wave_out[i];
    }


/* --------------   do ramping / overlapped writing out ---------------- */
/*  */if(verbose)printf("writing out\n");
    if(first){ /* write out all data before ramp */
      fwrite(&bufout[1],SHORTS,ENDPTS,fd_out);
               /* ramp down last NRAMP points */
      for (i = 1; i <= NRAMP ; i++) ramp[i] = bufout[ENDPTS+i]*(float)(NRAMP-i)/(float)NRAMP;
printf("first written %d first = %d, last = %d\n",ENDPTS,first,last);
      first = FALSE;
printf("first written %d first = %d, last = %d\n",ENDPTS,first,last);
    }
/*--------------------------------------------------------------------*/

    else if(!first){
                     /* ramp up 1st NRAMP points, adding to last points of previous buffer */
                     /* and write */
      for (i = 1; i <= NRAMP ; i++) ramp[i] += bufout[i]*(float)(i)/(float)NRAMP;
      fwrite(&ramp[1],SHORTS,NRAMP,fd_out);
fprintf(stderr,"block %ld: (!first) written %d\n",block,NRAMP);

      if(!last){           /* write out middle unramped points */
        fwrite(&bufout[NRAMP+1],SHORTS,NWRITE,fd_out);
fprintf(stderr,"(!last ) written %d\n",NWRITE);
                         /* put last NRAMP points into ramp[] to be added to next buffer */
        for (i = 1; i <= NRAMP ; i++) ramp[i] = bufout[ENDPTS+i]*(float)(1+NRAMP-i)/(float)NRAMP;
      }
      else if(last){
        fwrite(&bufout[NRAMP+1],SHORTS,last_pts,fd_out);
fprintf(stderr,"( last ) written %ld\n",last_pts);
      }/* end of if(!last) else if(last) */
		}/* from-  else if(!first) */
fprintf(stderr,"-------block %ld written-----------\n",block);
	}/* while (!last) */
  printf("closing file\n");
  fclose(fd_out);
}
