/*              recwide4.h

    for 3 times wide filter bank/recruitment simulation
    defines for modular recursive filterbank system
    high pass filter for top 5 channels (8 - 12)
originally M.A.Stone Aug 1990. ('normal' filter bank)
modified B.R.Glasberg 1993-1995 recruitment simulation
*/

#include <math.h>
#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>

#define Pi    3.141592654
#define TwoPi (2.*Pi)
#define BROADENING 3.00   /* broadening factor relative to 'normal' 'erbs' */
/*
   100 Hz, E = 3.36, 5836.7 Hz, E = 30.36, thus 27 erbs cover 13 channels
   therefore spacing = 2.25 (27/12) erbs/channel. Since broadening is 3,
   the channels are spaced at 0.75 broadened erb
*/
#define N_OF_CHANS 13
#define LOWEST_CF 100.0
#define HIGHEST_CF 5836.7
#define SAMPLERATE 16000

#define SAMPLE_CLK (float)SAMPLERATE
#define K_RATE (SAMPLE_CLK/1000.) /* sample rate in kilohertz */
#define BUFFSIZE 4096   /* how many samples to process: always power of 2. */
#define OUTBUFSIZE (N_OF_CHANS * BUFFSIZE)
#define D_BLOCK BUFFSIZE
#define SUBSET (BUFFSIZE/4) /*size of sub-loop in processing: reduces array sizes*/
#define NGAMMA 4  /* order of gammatone filter */
#define DELAY_SF (float)(NGAMMA-1)/* delay scale factor to time align FB output */
#define FILTER_ORDER 4    /* 4th order recursive filter for filter bank */
#define NCFS 4    /* 4 coefficients per recursive filter stage in filter bank */
#define XiYj         9    /* number of states stored for each filter vector */
#define MAX_MOD 100.0     /* max modulation rate for activity signal (-3 dB) */
#define SHORTS sizeof(short)
#define NULLPTR NULL
#define YES 1 /* flags for whether to write biquad filterbank data */
#define NO 0

/* ------ new #defines for hpbessel high pass filter additional bit ------- */

#define HPSTRT 8 /* channel number at which to start using the high-pass filters */
#define No_to_HPass 5 /* number of channels to tack on additional hp filter */
#define N_HPCFS 5 /* number of coefficients per HP filter */
#define N_HPSTS 2 /* number of states to be stored for each HP filter */

static float iircfs[N_HPCFS*No_to_HPass]; /* these should not really be placed here..... */
static float iirvect[N_HPSTS*No_to_HPass];/* .......because compilers tend to whinge ! */

/* ------ end of new #defines ------- */


/* private data structure for general biquadratic filter section */

struct Biquad {
	float k0num, k1num;     /* numerator coefficients */
	float k1denom, k2denom; /* denominator coefficients */
	float E_no;             /* filter cf on E scale */
	float a1,b1;    /* activity signal filter coefficients */
	long delay;     /* group delay of filter in integer samples */
};

struct Hpbessel {
	float a0,a1,a2; /* filter coefficients */
	float b1,b2; /* b0 defaults to unity */
};

/* general data structure for agc action: allow for RC attack & decay,
	with variable time const, include space for optional holdoff */

typedef float Filter_State; /* create new type of pointer to filter state */
typedef struct {
	float re,im;
} complex;
